" Vim indent file
" Language:	    ProvideX v5.14
" Maintainer:	    Tyler Harsh <harsht@lantard.com>
" Latest Revision:  12-7-2012

" Only load this indent file when no other was loaded.
if exists("b:did_indent")
   finish
endif
let b:did_indent = 1

setlocal indentexpr=GetPvxIndent()
" The end will signal an end_if, the def will signal a default also.
setlocal indentkeys+=0=then,0=else,0=def,{,},\,<!>,0=rem,0=switch,0=case,0=break
setlocal indentkeys+=0=select,0=for,0=next,0=while,0=wend,0=repeat,0=until
setlocal indentkeys+=0=return,0=exit,0=end,0=retry
setlocal nosmartindent

" Only define the function once.
if exists("*GetPvxIndent")
   finish
endif

function! GetPvxIndent()
   " Find a non-blank line above the current line.
   let lnum = prevnonblank(v:lnum - 1)
   " Hit the start of the file, use zero indent.
   if lnum == 0
      return 0
   endif

   let ind = indent(lnum)
   let pline = getline(lnum)
   let line = getline(v:lnum)
   " Find all Increasing
   "  If you define a 1-line function it shouldn't indent the line after it.
   "  Labels can have comments after it but Line Continuations (\) cannot.
   if pline =~ '^\s*\(switch\|case\|default\|while\|for\|repeat\|select\)\>' 
	    \ || pline =~ '^\s*\w\+:\s*\(\(!\|rem\).*\)\?$' || pline =~ '=\s\+\\\s*$'
	    \ || pline =~ '^\s*\(if\|local\).*\\\s*$' || pline =~ '^\s*def\s\+\(fn\|class\)[^=]\+$' 
	    \ || pline =~ '^\s*\(!\|}\s*else\s*\)\?{' || pline =~ '{\s*\(\(!\|rem\).*\)\?$'
	    \ || pline =~ '^\s*else\s*\(\(!\|rem\).*\|\\\s*\)\?$' 
	    \ || pline =~ '^\s*function\s\+.*)\s*\(\(!\|rem\).*\)\?$' 
      let ind = ind + &sw
   endif

   " Find all Decreasing 
   " Needs to be separate so it does 2 decreases if needed
   if pline =~ '^\s*break\>'
	    \ || pline =~ '^\s*[^}]*\\\s*$' && line =~ '^\s*else\s*\\\s*$'
      let ind = ind - &sw
   endif
   if line =~ '^\s*\(end\s\+switch\|function\s\+end\|end\s\+def\|next\|wend\|until\)\>' 
	    \ || line =~ '^\s*\([{}]\|!}\)'
	    \ || line =~ '^\s*else\s*\(\(!\|rem\).*\)\?$' 
      let ind = ind - &sw
   endif

   " Force 0 Indent If line starts with "!!" or only is a special word
   " Makes it so a decrease doesn't go before the beg of a line.  
   if line =~ '^\s*\<\(return\|exit\|end\|retry\)\>\s*$'
	    \ || line =~ '^\s*!!' || ind < 0
      let ind = 0
   endif

   return ind
endfunction

