" Vim syntax file
" Language: ProvideX v5.14
" Vim Version: Vim63
" Maintainer: Tyler Harsh <harsht@lantard.com>
" Last Change: 2012 Nov 11

if version < 600
   syntax clear
elseif exists("b:current_syntax")
   finish
endif

" ProvideX is case insensitive
syn case ignore

" ProvideX Directives - Only or First word
syn keyword pvxDirective ACCEPT ADDR AUTO BEGIN BUTTON BYE CALL
syn keyword pvxDirective CHART CHECK_BOX CLEAR CLIP_BOARD CLOSE
syn keyword pvxDirective CUSTOM_VBX CWDIR DATA DAY_FORMAT
syn keyword pvxDirective DEFCTL DEFPRT DEFTTY DICTIONARY
syn keyword pvxDirective DIM DIRECT DIRECTORY DISABLE
syn keyword pvxDirective DROP DROP_BOX DUMP EDIT ENABLE END
syn keyword pvxDirective ENDTRACE ENTER ERASE ERROR_HANDLER ESCAPE
syn keyword pvxDirective EXECUTE EXIT EXTRACT FILE FIND GET_FILE_BOX
syn keyword pvxDirective GRID H_SCROLLBAR HIDE INDEXED
syn keyword pvxDirective INPUT INSERT INVOKE IOLIST KEYED LET LIKE
syn keyword pvxDirective LINE_SWITCH LIST LIST_BOX LOAD LOCAL LOCK
syn keyword pvxDirective LONG_FORM MENU_BAR MERGE MESSAGE_LIB MNEMONIC
syn keyword pvxDirective MSGBOX MULTI_LINE MULTI_MEDIA OBJECT OBTAIN ON OPEN
syn keyword pvxDirective PASSWORD PERFORM POP POPUP_MENU PRECISION PREFIX
syn keyword pvxDirective PREINPUT PRINT PROCESS PROGRAM PROPERTY PURGE
syn keyword pvxDirective QUIT RADIO_BUTTON RANDOMIZE READ REDIM REFILE
syn keyword pvxDirective RELEASE REMOVE RENAME RENUMBER RESET
syn keyword pvxDirective RETRY RETURN ROUND RUN SAVE SERIAL
syn keyword pvxDirective SET_FOCUS SET_NBF SET_PARAM SETDAY SETDEV
syn keyword pvxDirective SETDRIVE SETFID SETMOUSE SETTIME
syn keyword pvxDirective SETTRACE SHORT_FORM SHOW SORT START STATIC STOP
syn keyword pvxDirective SYSTEM_HELP SYSTEM_JRNL TABLE TRANSLATE
syn keyword pvxDirective TRISTATE_BOX UNLOCK UPDATE USER_LEX
syn keyword pvxDirective V_SCROLLBAR VARDROP_BOX VARLIST_BOX VIA
syn keyword pvxDirective VIDEO_PALETTE WAIT WINDOW WINPRT_SETUP WRITE
" Providex Directives - Not the First word or have a space
syn keyword pvxDirective ADD INDEX  DEF CLASS FLOATING POINT
syn keyword pvxDirective CONTROL  RECORD  SERVER  EXCEPT
syn keyword pvxDirective UNIQUE  REQUIRED 

" Providex Conditionals
syn keyword pvxConditionals IF THEN ELSE END_IF FI SWITCH CASE BREAK
syn keyword pvxConditionals DEFAULT OR WHERE TO FROM

" Providex Repeat/Loops
syn keyword pvxLoops FOR NEXT WHILE WEND REPEAT UNTIL SELECT STEP

" Providex Arguments
syn keyword pvxArguments KNO RTY SIZ OFF ISZ TIP FNT MNU

" ProvideX Functions - All these have a ( after them
syn keyword pvxFunction ABS ACS AND ARG ASC ASN ATH ATN BIN BSZ CHG CHR COS
syn keyword pvxFunction CPL CRC CSE CVS DEC DIM DIR DLL DLX DSK DTE ENV
syn keyword pvxFunction EPT EVN EVS EXP FFN FIB FID FIN FPT GAP GBL GEP
syn keyword pvxFunction HSA HSH HTA HWN I3E IND INT IOL IOR JUL KEC KEF KEL
syn keyword pvxFunction KEN KEP KEY KGN LCS LEN LOG LRC LST MAX MEM MID MIN
syn keyword pvxFunction MNM MOD MSG MSK MXC MXL NEW NOT NUL NUM OBJ OPT PAD
syn keyword pvxFunction PCK PGM POS PTH PUB RCD RDX REC REF
syn keyword pvxFunction RNO SGN SIN SQR SRT SSZ STK STP STR SUB SWP
syn keyword pvxFunction TAN TBL TCB TMR TRX TSK TXH TXW UCS UPK VIN VIS XEQ
syn keyword pvxFunction XFA XOR

" System Variables
syn keyword pvxSysVariables BKG CHN DAY DLM DSZ EOM ERS ESC GFN GID HFN
syn keyword pvxSysVariables HLP HWD LFA LFO LIP LPG LWD MSE MSL NAR NID PGN
syn keyword pvxSysVariables PSZ QUO RET SID SSN TIM TME TMS
syn keyword pvxSysVariables TSM UID UNT WHO

" Contained Section
" ==============================================================
" Object Oriented
syn match pvxObjectFunction contained ".\{-1,}(.\{-})" contains=pvxDirective,pvxConditionals,pvxLoops,pvxArguments,pvxFunction,pvxSysVariables,pvxVariables,pvxNumVariables,pvxOperator,pvxOperatorMath nextgroup=pvxLabelRef

" Path Modifiers - [???] and anything else.
syn match pvxPathModifier contained "\[\a\a\a\]"
syn match pvxPathModifier contained "\*\a\+\*\?"

" Misc
syn match pvxVarValid contained ":(.\{-1,})" contains=pvxString,pvxLabelRef,pvxOperator
syn match pvxLabelStringVar contained ".\{-1,}" contains=pvxString,pvxVariables,pvxLabelRef
syn match pvxSetctl contained ".\{-1,}:" contains=pvxNumber,pvxNumVariables nextgroup=pvxLabelRef

" Labels
syn match pvxLabelRef contained "\h\w*\>" contains=pvxDirective,pvxConditionals,pvxLoops,pvxArguments,pvxFunction,pvxSysVariables
syn match pvxLabelRef contained "\<\d\+\>" 
syn match pvxLabelRef contained "\*\(break\|continue\|end\|escape\|next\|proceed\|retry\|return\|same\)" contains=pvxOperatorMath
syn match pvxLabelGo contained "\*\?\w\{-1,},"me=e-1 contains=pvxLabelRef nextgroup=pvxLabelGoOp
syn match pvxLabelGoOp contained "," nextgroup=pvxLabelGo,pvxLabelRef

" Comments
syn keyword pvxTodo contained FIXME TODO XXX
syn match pvxTodo contained "!!!"
syn keyword pvxCrit contained CRITICAL
syn match pvxCrit contained "\*\*\*"
syn match pvxCrit contained "???"
syn match pvxSpecialComment contained "\h\(\.\|\w\|-\)*\s*:"me=e-1
" ==============================================================

" Unused in V5.14 - Some may have no highlighting.  Here for lowest priority
" ----------------------------------------------
" Comments
syn region pvxComment start="!\*" end="\*!" contains=pvxTodo,pvxCrit,pvxSpecialComment
syn region pvxDocumentation start="^=begin" end="^=end.*$" contains=pvxTodo,pvxCrit,pvxSpecialComment fold
" ----------------------------------------------

" Variables
syn match pvxNumVariables "[%]\?\h\w*[%]\?"
syn match pvxVariables "[%]\?\h\w*[\$\.&!#]" nextgroup=pvxVarValid
syn match pvxFn "fn" nextgroup=pvxVariables,pvxNumVariables

" Misc
syn match pvxOperatorMath "[\*+\-/^]"
syn match pvxOperator "[|\\'\.:;=]"
syn match pvxOperator "[<>(){}\[\]]"
syn match pvxOperator "@[XY]\?"
syn region pvxNewline start="\\" end="$" contains=pvxComment keepend

" Strings, Constants and Numbers
syn region pvxString start=+"+ end=+"+ contains=pvxPathModifier 
syn match pvxHex "\$\(\x\|\s\)*\$"
syn match pvxMnemonic "'[0-9A-Za-z_@+\-\*?]*'"
syn match pvxNumber "\.\?\<\(\d\|\.\)\+\>"

" Labels
syn match pvxLineNumber "^\d\+\s"me=e-1
syn match pvxLabel "^\(\<\d*\>\)\?\s*\h\w*\s*:"me=e-1
syn match pvxLabel "\<continue\>"

" Comments
syn region pvxComment start="\(!\|\<rem\>\)" end="$" contains=pvxTodo,pvxCrit,pvxSpecialComment

" Keywords with Labels 
syn match pvxDirective "\<\(setesc\|seterr\)\s\+" nextgroup=pvxLabelStringVar
syn match pvxDirective "\<setctl\s\+" nextgroup=pvxSetctl
syn match pvxDirective "\<restore\s\+" nextgroup=pvxLabelRef
syn match pvxDirective "\<\(gosub\|goto\|delete\|create\)\s\+" nextgroup=pvxLabelGo,pvxLabelRef
syn match pvxConditionals "\<exitto\s\+" nextgroup=pvxLabelRef
syn match pvxArguments "\(\s\|)\|:\)iol=" contains=pvxOperator nextgroup=pvxLabelRef
syn match pvxArguments ",\s*\(end\|err\|nul\|bsy\|tbl\|dom\|iol\)="hs=s+1 contains=pvxOperator nextgroup=pvxVariables,pvxLabelRef

" SysVariables that are Functions or Arguments also
syn match pvxSysVariables "\<\(err\|prc\|ctl\|pfx\|prm\|rnd\|sep\|sys\)\>"
syn match pvxFunction "\<\(err\|prc\|ctl\|pfx\|prm\|rnd\|sep\|sys\)\s*("me=e-1
syn match pvxArguments ",\s*\<\(tim\|sep\)\>="hs=s+1,me=e-1

" Special Cases
syn match pvxDirective "\<delete\s\+object\>"
syn match pvxDirective "\<setesc\s\+\(on\|off\|disable\|enable\)\>"
syn match pvxArguments "{all}"

" Object Oriented Stuff
syn match pvxDirective "\<function\s\+\(\(perform\|local\)\s\+\)\?" nextgroup=pvxObjectFunction
syn match pvxArguments "\<\(get\|set\)\s\+\(err\)\?" nextgroup=pvxLabelRef



if version >= 508 || !exists("did_basic_syntax_inits")
   if version < 508
      let did_basic_syntax_inits = 1
      command -nargs=+ HiLink hi link <args>
   else
      command -nargs=+ HiLink hi def link <args>
   endif

   " Colors (Providex Uses 7 different colors for stuff below)
   " 1: Background colour for highlighting *[...] searches *Not used here
   " 2: Colour for Variables
   " 3: Colour for Literals
   " 4: Colour for Remarks
   " 5: Colour for Error Lines
   " 6: Colour for Mnemonics
   " 7: Colour for Statement #s or Labels
   HiLink pvxArguments Keyword
   HiLink pvxComment Comment
   HiLink pvxConditionals Conditional
   HiLink pvxDirective Keyword
   HiLink pvxFunction Function
   HiLink pvxFn Function
   HiLink pvxHex Constant
   HiLink pvxLabel Label
   HiLink pvxLabelArg Keyword
   HiLink pvxLabelRef Label
   HiLink pvxLabelGo Label
   HiLink pvxLineNumber Statement
   HiLink pvxLoops Repeat
   HiLink pvxMnemonic SpecialChar
   HiLink pvxNewline PreCondit
   HiLink pvxNumber Number
   " NumIdentifier is unique.  Can be changed to Identifier
   HiLink pvxNumVariables NumIdentifier
   HiLink pvxObjectFunction Keyword
   HiLink pvxOperator Operator
   HiLink pvxOperatorMath Operator
   HiLink pvxPathModifier Constant
   HiLink pvxSpecialComment SpecialComment
   HiLink pvxString String
   HiLink pvxSysVariables Special
   HiLink pvxTodo Todo
   HiLink pvxVariables Identifier
   HiLink pvxCrit Error

   delcommand HiLink
endif

let b:current_syntax = "providex"
